@@ -1,4 +1,4 @@
-/* A Bison parser, made by GNU Bison 3.8.2.  */
+/* A Bison parser, made by GNU Bison 3.7.6.  */
 
 /* Bison interface for Yacc-like parsers in C
 
@@ -342,74 +342,83 @@ extern int yydebug;
     VAR_CACHEDB_REDISPATH = 543,   /* VAR_CACHEDB_REDISPATH  */
     VAR_CACHEDB_REDISPASSWORD = 544, /* VAR_CACHEDB_REDISPASSWORD  */
     VAR_CACHEDB_REDISLOGICALDB = 545, /* VAR_CACHEDB_REDISLOGICALDB  */
-    VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM = 546, /* VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM  */
-    VAR_FOR_UPSTREAM = 547,        /* VAR_FOR_UPSTREAM  */
-    VAR_AUTH_ZONE = 548,           /* VAR_AUTH_ZONE  */
-    VAR_ZONEFILE = 549,            /* VAR_ZONEFILE  */
-    VAR_MASTER = 550,              /* VAR_MASTER  */
-    VAR_URL = 551,                 /* VAR_URL  */
-    VAR_FOR_DOWNSTREAM = 552,      /* VAR_FOR_DOWNSTREAM  */
-    VAR_FALLBACK_ENABLED = 553,    /* VAR_FALLBACK_ENABLED  */
-    VAR_TLS_ADDITIONAL_PORT = 554, /* VAR_TLS_ADDITIONAL_PORT  */
-    VAR_LOW_RTT = 555,             /* VAR_LOW_RTT  */
-    VAR_LOW_RTT_PERMIL = 556,      /* VAR_LOW_RTT_PERMIL  */
-    VAR_FAST_SERVER_PERMIL = 557,  /* VAR_FAST_SERVER_PERMIL  */
-    VAR_FAST_SERVER_NUM = 558,     /* VAR_FAST_SERVER_NUM  */
-    VAR_ALLOW_NOTIFY = 559,        /* VAR_ALLOW_NOTIFY  */
-    VAR_TLS_WIN_CERT = 560,        /* VAR_TLS_WIN_CERT  */
-    VAR_TCP_CONNECTION_LIMIT = 561, /* VAR_TCP_CONNECTION_LIMIT  */
-    VAR_ANSWER_COOKIE = 562,       /* VAR_ANSWER_COOKIE  */
-    VAR_COOKIE_SECRET = 563,       /* VAR_COOKIE_SECRET  */
-    VAR_IP_RATELIMIT_COOKIE = 564, /* VAR_IP_RATELIMIT_COOKIE  */
-    VAR_FORWARD_NO_CACHE = 565,    /* VAR_FORWARD_NO_CACHE  */
-    VAR_STUB_NO_CACHE = 566,       /* VAR_STUB_NO_CACHE  */
-    VAR_LOG_SERVFAIL = 567,        /* VAR_LOG_SERVFAIL  */
-    VAR_DENY_ANY = 568,            /* VAR_DENY_ANY  */
-    VAR_UNKNOWN_SERVER_TIME_LIMIT = 569, /* VAR_UNKNOWN_SERVER_TIME_LIMIT  */
-    VAR_LOG_TAG_QUERYREPLY = 570,  /* VAR_LOG_TAG_QUERYREPLY  */
-    VAR_DISCARD_TIMEOUT = 571,     /* VAR_DISCARD_TIMEOUT  */
-    VAR_WAIT_LIMIT = 572,          /* VAR_WAIT_LIMIT  */
-    VAR_WAIT_LIMIT_COOKIE = 573,   /* VAR_WAIT_LIMIT_COOKIE  */
-    VAR_WAIT_LIMIT_NETBLOCK = 574, /* VAR_WAIT_LIMIT_NETBLOCK  */
-    VAR_WAIT_LIMIT_COOKIE_NETBLOCK = 575, /* VAR_WAIT_LIMIT_COOKIE_NETBLOCK  */
-    VAR_STREAM_WAIT_SIZE = 576,    /* VAR_STREAM_WAIT_SIZE  */
-    VAR_TLS_CIPHERS = 577,         /* VAR_TLS_CIPHERS  */
-    VAR_TLS_CIPHERSUITES = 578,    /* VAR_TLS_CIPHERSUITES  */
-    VAR_TLS_USE_SNI = 579,         /* VAR_TLS_USE_SNI  */
-    VAR_IPSET = 580,               /* VAR_IPSET  */
-    VAR_IPSET_NAME_V4 = 581,       /* VAR_IPSET_NAME_V4  */
-    VAR_IPSET_NAME_V6 = 582,       /* VAR_IPSET_NAME_V6  */
-    VAR_TLS_SESSION_TICKET_KEYS = 583, /* VAR_TLS_SESSION_TICKET_KEYS  */
-    VAR_RPZ = 584,                 /* VAR_RPZ  */
-    VAR_TAGS = 585,                /* VAR_TAGS  */
-    VAR_RPZ_ACTION_OVERRIDE = 586, /* VAR_RPZ_ACTION_OVERRIDE  */
-    VAR_RPZ_CNAME_OVERRIDE = 587,  /* VAR_RPZ_CNAME_OVERRIDE  */
-    VAR_RPZ_LOG = 588,             /* VAR_RPZ_LOG  */
-    VAR_RPZ_LOG_NAME = 589,        /* VAR_RPZ_LOG_NAME  */
-    VAR_DYNLIB = 590,              /* VAR_DYNLIB  */
-    VAR_DYNLIB_FILE = 591,         /* VAR_DYNLIB_FILE  */
-    VAR_EDNS_CLIENT_STRING = 592,  /* VAR_EDNS_CLIENT_STRING  */
-    VAR_EDNS_CLIENT_STRING_OPCODE = 593, /* VAR_EDNS_CLIENT_STRING_OPCODE  */
-    VAR_NSID = 594,                /* VAR_NSID  */
-    VAR_ZONEMD_PERMISSIVE_MODE = 595, /* VAR_ZONEMD_PERMISSIVE_MODE  */
-    VAR_ZONEMD_CHECK = 596,        /* VAR_ZONEMD_CHECK  */
-    VAR_ZONEMD_REJECT_ABSENCE = 597, /* VAR_ZONEMD_REJECT_ABSENCE  */
-    VAR_RPZ_SIGNAL_NXDOMAIN_RA = 598, /* VAR_RPZ_SIGNAL_NXDOMAIN_RA  */
-    VAR_INTERFACE_AUTOMATIC_PORTS = 599, /* VAR_INTERFACE_AUTOMATIC_PORTS  */
-    VAR_EDE = 600,                 /* VAR_EDE  */
-    VAR_INTERFACE_ACTION = 601,    /* VAR_INTERFACE_ACTION  */
-    VAR_INTERFACE_VIEW = 602,      /* VAR_INTERFACE_VIEW  */
-    VAR_INTERFACE_TAG = 603,       /* VAR_INTERFACE_TAG  */
-    VAR_INTERFACE_TAG_ACTION = 604, /* VAR_INTERFACE_TAG_ACTION  */
-    VAR_INTERFACE_TAG_DATA = 605,  /* VAR_INTERFACE_TAG_DATA  */
-    VAR_PROXY_PROTOCOL_PORT = 606, /* VAR_PROXY_PROTOCOL_PORT  */
-    VAR_STATISTICS_INHIBIT_ZERO = 607, /* VAR_STATISTICS_INHIBIT_ZERO  */
-    VAR_HARDEN_UNKNOWN_ADDITIONAL = 608, /* VAR_HARDEN_UNKNOWN_ADDITIONAL  */
-    VAR_DISABLE_EDNS_DO = 609,     /* VAR_DISABLE_EDNS_DO  */
-    VAR_CACHEDB_NO_STORE = 610,    /* VAR_CACHEDB_NO_STORE  */
-    VAR_LOG_DESTADDR = 611,        /* VAR_LOG_DESTADDR  */
-    VAR_CACHEDB_CHECK_WHEN_SERVE_EXPIRED = 612, /* VAR_CACHEDB_CHECK_WHEN_SERVE_EXPIRED  */
-    VAR_COOKIE_SECRET_FILE = 613   /* VAR_COOKIE_SECRET_FILE  */
+    VAR_CACHEDB_REDISCOMMANDTIMEOUT = 546, /* VAR_CACHEDB_REDISCOMMANDTIMEOUT  */
+    VAR_CACHEDB_REDISCONNECTTIMEOUT = 547, /* VAR_CACHEDB_REDISCONNECTTIMEOUT  */
+    VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM = 548, /* VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM  */
+    VAR_FOR_UPSTREAM = 549,        /* VAR_FOR_UPSTREAM  */
+    VAR_AUTH_ZONE = 550,           /* VAR_AUTH_ZONE  */
+    VAR_ZONEFILE = 551,            /* VAR_ZONEFILE  */
+    VAR_MASTER = 552,              /* VAR_MASTER  */
+    VAR_URL = 553,                 /* VAR_URL  */
+    VAR_FOR_DOWNSTREAM = 554,      /* VAR_FOR_DOWNSTREAM  */
+    VAR_FALLBACK_ENABLED = 555,    /* VAR_FALLBACK_ENABLED  */
+    VAR_TLS_ADDITIONAL_PORT = 556, /* VAR_TLS_ADDITIONAL_PORT  */
+    VAR_LOW_RTT = 557,             /* VAR_LOW_RTT  */
+    VAR_LOW_RTT_PERMIL = 558,      /* VAR_LOW_RTT_PERMIL  */
+    VAR_FAST_SERVER_PERMIL = 559,  /* VAR_FAST_SERVER_PERMIL  */
+    VAR_FAST_SERVER_NUM = 560,     /* VAR_FAST_SERVER_NUM  */
+    VAR_ALLOW_NOTIFY = 561,        /* VAR_ALLOW_NOTIFY  */
+    VAR_TLS_WIN_CERT = 562,        /* VAR_TLS_WIN_CERT  */
+    VAR_TCP_CONNECTION_LIMIT = 563, /* VAR_TCP_CONNECTION_LIMIT  */
+    VAR_ANSWER_COOKIE = 564,       /* VAR_ANSWER_COOKIE  */
+    VAR_COOKIE_SECRET = 565,       /* VAR_COOKIE_SECRET  */
+    VAR_IP_RATELIMIT_COOKIE = 566, /* VAR_IP_RATELIMIT_COOKIE  */
+    VAR_FORWARD_NO_CACHE = 567,    /* VAR_FORWARD_NO_CACHE  */
+    VAR_STUB_NO_CACHE = 568,       /* VAR_STUB_NO_CACHE  */
+    VAR_LOG_SERVFAIL = 569,        /* VAR_LOG_SERVFAIL  */
+    VAR_DENY_ANY = 570,            /* VAR_DENY_ANY  */
+    VAR_UNKNOWN_SERVER_TIME_LIMIT = 571, /* VAR_UNKNOWN_SERVER_TIME_LIMIT  */
+    VAR_LOG_TAG_QUERYREPLY = 572,  /* VAR_LOG_TAG_QUERYREPLY  */
+    VAR_DISCARD_TIMEOUT = 573,     /* VAR_DISCARD_TIMEOUT  */
+    VAR_WAIT_LIMIT = 574,          /* VAR_WAIT_LIMIT  */
+    VAR_WAIT_LIMIT_COOKIE = 575,   /* VAR_WAIT_LIMIT_COOKIE  */
+    VAR_WAIT_LIMIT_NETBLOCK = 576, /* VAR_WAIT_LIMIT_NETBLOCK  */
+    VAR_WAIT_LIMIT_COOKIE_NETBLOCK = 577, /* VAR_WAIT_LIMIT_COOKIE_NETBLOCK  */
+    VAR_STREAM_WAIT_SIZE = 578,    /* VAR_STREAM_WAIT_SIZE  */
+    VAR_TLS_CIPHERS = 579,         /* VAR_TLS_CIPHERS  */
+    VAR_TLS_CIPHERSUITES = 580,    /* VAR_TLS_CIPHERSUITES  */
+    VAR_TLS_USE_SNI = 581,         /* VAR_TLS_USE_SNI  */
+    VAR_IPSET = 582,               /* VAR_IPSET  */
+    VAR_IPSET_NAME_V4 = 583,       /* VAR_IPSET_NAME_V4  */
+    VAR_IPSET_NAME_V6 = 584,       /* VAR_IPSET_NAME_V6  */
+    VAR_TLS_SESSION_TICKET_KEYS = 585, /* VAR_TLS_SESSION_TICKET_KEYS  */
+    VAR_RPZ = 586,                 /* VAR_RPZ  */
+    VAR_TAGS = 587,                /* VAR_TAGS  */
+    VAR_RPZ_ACTION_OVERRIDE = 588, /* VAR_RPZ_ACTION_OVERRIDE  */
+    VAR_RPZ_CNAME_OVERRIDE = 589,  /* VAR_RPZ_CNAME_OVERRIDE  */
+    VAR_RPZ_LOG = 590,             /* VAR_RPZ_LOG  */
+    VAR_RPZ_LOG_NAME = 591,        /* VAR_RPZ_LOG_NAME  */
+    VAR_DYNLIB = 592,              /* VAR_DYNLIB  */
+    VAR_DYNLIB_FILE = 593,         /* VAR_DYNLIB_FILE  */
+    VAR_EDNS_CLIENT_STRING = 594,  /* VAR_EDNS_CLIENT_STRING  */
+    VAR_EDNS_CLIENT_STRING_OPCODE = 595, /* VAR_EDNS_CLIENT_STRING_OPCODE  */
+    VAR_NSID = 596,                /* VAR_NSID  */
+    VAR_ZONEMD_PERMISSIVE_MODE = 597, /* VAR_ZONEMD_PERMISSIVE_MODE  */
+    VAR_ZONEMD_CHECK = 598,        /* VAR_ZONEMD_CHECK  */
+    VAR_ZONEMD_REJECT_ABSENCE = 599, /* VAR_ZONEMD_REJECT_ABSENCE  */
+    VAR_RPZ_SIGNAL_NXDOMAIN_RA = 600, /* VAR_RPZ_SIGNAL_NXDOMAIN_RA  */
+    VAR_INTERFACE_AUTOMATIC_PORTS = 601, /* VAR_INTERFACE_AUTOMATIC_PORTS  */
+    VAR_EDE = 602,                 /* VAR_EDE  */
+    VAR_INTERFACE_ACTION = 603,    /* VAR_INTERFACE_ACTION  */
+    VAR_INTERFACE_VIEW = 604,      /* VAR_INTERFACE_VIEW  */
+    VAR_INTERFACE_TAG = 605,       /* VAR_INTERFACE_TAG  */
+    VAR_INTERFACE_TAG_ACTION = 606, /* VAR_INTERFACE_TAG_ACTION  */
+    VAR_INTERFACE_TAG_DATA = 607,  /* VAR_INTERFACE_TAG_DATA  */
+    VAR_QUIC_PORT = 608,           /* VAR_QUIC_PORT  */
+    VAR_QUIC_SIZE = 609,           /* VAR_QUIC_SIZE  */
+    VAR_PROXY_PROTOCOL_PORT = 610, /* VAR_PROXY_PROTOCOL_PORT  */
+    VAR_STATISTICS_INHIBIT_ZERO = 611, /* VAR_STATISTICS_INHIBIT_ZERO  */
+    VAR_HARDEN_UNKNOWN_ADDITIONAL = 612, /* VAR_HARDEN_UNKNOWN_ADDITIONAL  */
+    VAR_DISABLE_EDNS_DO = 613,     /* VAR_DISABLE_EDNS_DO  */
+    VAR_CACHEDB_NO_STORE = 614,    /* VAR_CACHEDB_NO_STORE  */
+    VAR_LOG_DESTADDR = 615,        /* VAR_LOG_DESTADDR  */
+    VAR_CACHEDB_CHECK_WHEN_SERVE_EXPIRED = 616, /* VAR_CACHEDB_CHECK_WHEN_SERVE_EXPIRED  */
+    VAR_COOKIE_SECRET_FILE = 617,  /* VAR_COOKIE_SECRET_FILE  */
+    VAR_ITER_SCRUB_NS = 618,       /* VAR_ITER_SCRUB_NS  */
+    VAR_ITER_SCRUB_CNAME = 619,    /* VAR_ITER_SCRUB_CNAME  */
+    VAR_MAX_GLOBAL_QUOTA = 620,    /* VAR_MAX_GLOBAL_QUOTA  */
+    VAR_HARDEN_UNVERIFIED_GLUE = 621, /* VAR_HARDEN_UNVERIFIED_GLUE  */
+    VAR_LOG_TIME_ISO = 622         /* VAR_LOG_TIME_ISO  */
   };
   typedef enum yytokentype yytoken_kind_t;
 #endif
@@ -706,74 +715,83 @@ extern int yydebug;
 #define VAR_CACHEDB_REDISPATH 543
 #define VAR_CACHEDB_REDISPASSWORD 544
 #define VAR_CACHEDB_REDISLOGICALDB 545
-#define VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM 546
-#define VAR_FOR_UPSTREAM 547
-#define VAR_AUTH_ZONE 548
-#define VAR_ZONEFILE 549
-#define VAR_MASTER 550
-#define VAR_URL 551
-#define VAR_FOR_DOWNSTREAM 552
-#define VAR_FALLBACK_ENABLED 553
-#define VAR_TLS_ADDITIONAL_PORT 554
-#define VAR_LOW_RTT 555
-#define VAR_LOW_RTT_PERMIL 556
-#define VAR_FAST_SERVER_PERMIL 557
-#define VAR_FAST_SERVER_NUM 558
-#define VAR_ALLOW_NOTIFY 559
-#define VAR_TLS_WIN_CERT 560
-#define VAR_TCP_CONNECTION_LIMIT 561
-#define VAR_ANSWER_COOKIE 562
-#define VAR_COOKIE_SECRET 563
-#define VAR_IP_RATELIMIT_COOKIE 564
-#define VAR_FORWARD_NO_CACHE 565
-#define VAR_STUB_NO_CACHE 566
-#define VAR_LOG_SERVFAIL 567
-#define VAR_DENY_ANY 568
-#define VAR_UNKNOWN_SERVER_TIME_LIMIT 569
-#define VAR_LOG_TAG_QUERYREPLY 570
-#define VAR_DISCARD_TIMEOUT 571
-#define VAR_WAIT_LIMIT 572
-#define VAR_WAIT_LIMIT_COOKIE 573
-#define VAR_WAIT_LIMIT_NETBLOCK 574
-#define VAR_WAIT_LIMIT_COOKIE_NETBLOCK 575
-#define VAR_STREAM_WAIT_SIZE 576
-#define VAR_TLS_CIPHERS 577
-#define VAR_TLS_CIPHERSUITES 578
-#define VAR_TLS_USE_SNI 579
-#define VAR_IPSET 580
-#define VAR_IPSET_NAME_V4 581
-#define VAR_IPSET_NAME_V6 582
-#define VAR_TLS_SESSION_TICKET_KEYS 583
-#define VAR_RPZ 584
-#define VAR_TAGS 585
-#define VAR_RPZ_ACTION_OVERRIDE 586
-#define VAR_RPZ_CNAME_OVERRIDE 587
-#define VAR_RPZ_LOG 588
-#define VAR_RPZ_LOG_NAME 589
-#define VAR_DYNLIB 590
-#define VAR_DYNLIB_FILE 591
-#define VAR_EDNS_CLIENT_STRING 592
-#define VAR_EDNS_CLIENT_STRING_OPCODE 593
-#define VAR_NSID 594
-#define VAR_ZONEMD_PERMISSIVE_MODE 595
-#define VAR_ZONEMD_CHECK 596
-#define VAR_ZONEMD_REJECT_ABSENCE 597
-#define VAR_RPZ_SIGNAL_NXDOMAIN_RA 598
-#define VAR_INTERFACE_AUTOMATIC_PORTS 599
-#define VAR_EDE 600
-#define VAR_INTERFACE_ACTION 601
-#define VAR_INTERFACE_VIEW 602
-#define VAR_INTERFACE_TAG 603
-#define VAR_INTERFACE_TAG_ACTION 604
-#define VAR_INTERFACE_TAG_DATA 605
-#define VAR_PROXY_PROTOCOL_PORT 606
-#define VAR_STATISTICS_INHIBIT_ZERO 607
-#define VAR_HARDEN_UNKNOWN_ADDITIONAL 608
-#define VAR_DISABLE_EDNS_DO 609
-#define VAR_CACHEDB_NO_STORE 610
-#define VAR_LOG_DESTADDR 611
-#define VAR_CACHEDB_CHECK_WHEN_SERVE_EXPIRED 612
-#define VAR_COOKIE_SECRET_FILE 613
+#define VAR_CACHEDB_REDISCOMMANDTIMEOUT 546
+#define VAR_CACHEDB_REDISCONNECTTIMEOUT 547
+#define VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM 548
+#define VAR_FOR_UPSTREAM 549
+#define VAR_AUTH_ZONE 550
+#define VAR_ZONEFILE 551
+#define VAR_MASTER 552
+#define VAR_URL 553
+#define VAR_FOR_DOWNSTREAM 554
+#define VAR_FALLBACK_ENABLED 555
+#define VAR_TLS_ADDITIONAL_PORT 556
+#define VAR_LOW_RTT 557
+#define VAR_LOW_RTT_PERMIL 558
+#define VAR_FAST_SERVER_PERMIL 559
+#define VAR_FAST_SERVER_NUM 560
+#define VAR_ALLOW_NOTIFY 561
+#define VAR_TLS_WIN_CERT 562
+#define VAR_TCP_CONNECTION_LIMIT 563
+#define VAR_ANSWER_COOKIE 564
+#define VAR_COOKIE_SECRET 565
+#define VAR_IP_RATELIMIT_COOKIE 566
+#define VAR_FORWARD_NO_CACHE 567
+#define VAR_STUB_NO_CACHE 568
+#define VAR_LOG_SERVFAIL 569
+#define VAR_DENY_ANY 570
+#define VAR_UNKNOWN_SERVER_TIME_LIMIT 571
+#define VAR_LOG_TAG_QUERYREPLY 572
+#define VAR_DISCARD_TIMEOUT 573
+#define VAR_WAIT_LIMIT 574
+#define VAR_WAIT_LIMIT_COOKIE 575
+#define VAR_WAIT_LIMIT_NETBLOCK 576
+#define VAR_WAIT_LIMIT_COOKIE_NETBLOCK 577
+#define VAR_STREAM_WAIT_SIZE 578
+#define VAR_TLS_CIPHERS 579
+#define VAR_TLS_CIPHERSUITES 580
+#define VAR_TLS_USE_SNI 581
+#define VAR_IPSET 582
+#define VAR_IPSET_NAME_V4 583
+#define VAR_IPSET_NAME_V6 584
+#define VAR_TLS_SESSION_TICKET_KEYS 585
+#define VAR_RPZ 586
+#define VAR_TAGS 587
+#define VAR_RPZ_ACTION_OVERRIDE 588
+#define VAR_RPZ_CNAME_OVERRIDE 589
+#define VAR_RPZ_LOG 590
+#define VAR_RPZ_LOG_NAME 591
+#define VAR_DYNLIB 592
+#define VAR_DYNLIB_FILE 593
+#define VAR_EDNS_CLIENT_STRING 594
+#define VAR_EDNS_CLIENT_STRING_OPCODE 595
+#define VAR_NSID 596
+#define VAR_ZONEMD_PERMISSIVE_MODE 597
+#define VAR_ZONEMD_CHECK 598
+#define VAR_ZONEMD_REJECT_ABSENCE 599
+#define VAR_RPZ_SIGNAL_NXDOMAIN_RA 600
+#define VAR_INTERFACE_AUTOMATIC_PORTS 601
+#define VAR_EDE 602
+#define VAR_INTERFACE_ACTION 603
+#define VAR_INTERFACE_VIEW 604
+#define VAR_INTERFACE_TAG 605
+#define VAR_INTERFACE_TAG_ACTION 606
+#define VAR_INTERFACE_TAG_DATA 607
+#define VAR_QUIC_PORT 608
+#define VAR_QUIC_SIZE 609
+#define VAR_PROXY_PROTOCOL_PORT 610
+#define VAR_STATISTICS_INHIBIT_ZERO 611
+#define VAR_HARDEN_UNKNOWN_ADDITIONAL 612
+#define VAR_DISABLE_EDNS_DO 613
+#define VAR_CACHEDB_NO_STORE 614
+#define VAR_LOG_DESTADDR 615
+#define VAR_CACHEDB_CHECK_WHEN_SERVE_EXPIRED 616
+#define VAR_COOKIE_SECRET_FILE 617
+#define VAR_ITER_SCRUB_NS 618
+#define VAR_ITER_SCRUB_CNAME 619
+#define VAR_MAX_GLOBAL_QUOTA 620
+#define VAR_HARDEN_UNVERIFIED_GLUE 621
+#define VAR_LOG_TIME_ISO 622
 
 /* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
@@ -783,7 +801,7 @@ union YYSTYPE
 
 	char*	str;
 
-#line 787 "util/configparser.h"
+#line 805 "util/configparser.h"
 
 };
 typedef union YYSTYPE YYSTYPE;
@@ -794,8 +812,6 @@ typedef union YYSTYPE YYSTYPE;
 
 extern YYSTYPE yylval;
 
-
 int yyparse (void);
 
-
 #endif /* !YY_YY_UTIL_CONFIGPARSER_H_INCLUDED  */
