@@ -43,7 +43,7 @@
  */
 #include "config.h"
 #ifdef HAVE_OPENSSL_SSL_H
-#include "openssl/ssl.h"
+#include <openssl/ssl.h>
 #define NSEC3_SHA_LEN SHA_DIGEST_LENGTH
 #else
 #define NSEC3_SHA_LEN 20
@@ -823,7 +823,8 @@ void neg_insert_data(struct val_neg_cache* neg,
 			it <= neg->nsec3_max_iter &&
 			(h != zone->nsec3_hash || it != zone->nsec3_iter ||
 			slen != zone->nsec3_saltlen || 
-			memcmp(zone->nsec3_salt, s, slen) != 0)) {
+			(slen != 0 && zone->nsec3_salt && s
+			  && memcmp(zone->nsec3_salt, s, slen) != 0))) {
 
 			if(slen > 0) {
 				uint8_t* sa = memdup(s, slen);
@@ -1206,7 +1207,8 @@ neg_params_ok(struct val_neg_zone* zone, struct ub_packed_rrset_key* rrset)
 		return 0;
 	return (h == zone->nsec3_hash && it == zone->nsec3_iter &&
 		slen == zone->nsec3_saltlen &&
-		memcmp(zone->nsec3_salt, s, slen) == 0);
+		(slen != 0 && zone->nsec3_salt && s
+		  && memcmp(zone->nsec3_salt, s, slen) == 0));
 }
 
 /** get next closer for nsec3 proof */
@@ -1407,6 +1409,11 @@ val_neg_getmsg(struct val_neg_cache* neg, struct query_info* qinfo,
 	/* Matching NSEC, use to generate No Data answer. Not creating answers
 	 * yet for No Data proven using wildcard. */
 	if(nsec && nsec_proves_nodata(nsec, qinfo, &nodata_wc) && !nodata_wc) {
+		/* do not create nodata answers for qtype ANY, it is a query
+		 * type, not an rrtype to disprove. Nameerrors are useful for
+		 * qtype ANY, in the else branch. */
+		if(qinfo->qtype == LDNS_RR_TYPE_ANY)
+			return NULL;
 		if(!(msg = dns_msg_create(qinfo->qname, qinfo->qname_len, 
 			qinfo->qtype, qinfo->qclass, region, 2))) 
 			return NULL;
